# Generated by Django 5.0.6 on 2024-08-04 07:54

import django.db.models.deletion
import uuid
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='DjangoDataSeedBigIntegerModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('big_integer_field', models.BigIntegerField()),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedBinaryModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('binary_field', models.BinaryField()),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedBooleanModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('boolean_field', models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedCharModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('char_field', models.CharField(max_length=100)),
                ('choice_field', models.CharField(choices=[('option1', 'Option 1'), (
                    'option2', 'Option 2'), ('option3', 'Option 3')], default='option1', max_length=20)),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedDateModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('date_field', models.DateField()),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedDateTimeModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('date_time_field', models.DateTimeField()),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedDecimalModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('decimal_field', models.DecimalField(
                    decimal_places=2, max_digits=10)),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedDurationModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('duration_field', models.DurationField()),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedEmailModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('email_field', models.EmailField(max_length=254)),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedFloatModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('float_field', models.FloatField()),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedGenericIPAddressModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('generic_ip_address_field', models.GenericIPAddressField()),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedIntegerModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('integer_field', models.IntegerField()),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedIPAddressModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('ip_address_field', models.GenericIPAddressField()),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedJSONModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('json_field', models.JSONField(default=dict)),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedPositiveBigIntegerModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('positive_big_integer_field', models.PositiveBigIntegerField()),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedPositiveIntegerModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('positive_integer_field', models.PositiveIntegerField()),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedPositiveSmallIntegerModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('positive_small_integer_field',
                 models.PositiveSmallIntegerField()),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedSlugModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('slug_field', models.SlugField()),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedSmallIntegerModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('small_integer_field', models.SmallIntegerField()),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedTextModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('text_field', models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedTimeModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('time_field', models.TimeField()),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedURLModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('url_field', models.URLField()),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedUUIDModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid_field', models.UUIDField(default=uuid.uuid4, unique=True)),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedOneToOneModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid_field', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE,
                 related_name='DjangoDataSeedOneToOneModel_char_field', to='django_data_seed.djangodataseeduuidmodel')),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedManyToManyModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid_field', models.ManyToManyField(
                    related_name='DjangoDataSeedManyToManyModel_char_field', to='django_data_seed.djangodataseeduuidmodel')),
            ],
        ),
        migrations.CreateModel(
            name='DjangoDataSeedForeignKeyModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('integer_field', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                 related_name='DjangoDataSeedForeignKeyModel_integer_field', to='django_data_seed.djangodataseedintegermodel')),
                ('uuid_field', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                 related_name='DjangoDataSeedForeignKeyModel_char_field', to='django_data_seed.djangodataseeduuidmodel')),
            ],
        ),
        migrations.CreateModel(
            name='DjangoSeedDataBackUpModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('data', models.JSONField()),
                ('created_at', models.DateTimeField(auto_now_add=True, null=True)),
                ('updated_at', models.DateTimeField(auto_now=True, null=True)),
                ('object_id', models.CharField(
                    blank=True, max_length=100, null=True)),
                ('model_name', models.CharField(
                    blank=True, max_length=255, null=True)),
                ('deleted_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL,
                 related_name='DjangoSeedDataBackUpModel_deleted_by', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='DjangoSeedDataLogEntryModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('before_mutation', models.JSONField(blank=True, null=True)),
                ('after_mutation', models.JSONField(blank=True, null=True)),
                ('created_at', models.DateTimeField(auto_now_add=True, null=True)),
                ('updated_at', models.DateTimeField(auto_now=True, null=True)),
                ('object_id', models.CharField(
                    blank=True, max_length=100, null=True)),
                ('model_name', models.CharField(
                    blank=True, max_length=255, null=True)),
                ('mutated_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL,
                 related_name='DjangoSeedDataLogEntryModel_mutated_by', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]
